{========================================================================}
{=================  TVicHW16  component definition   ====================}
{==========             Freeware Version 1.21       =====================}
{==========     Copyright (c) 1997 Victor I.Ishikeev              =======}
{========================================================================}
{==========         mail to "victor@ivi.ugatu.ac.ru"              =======}
{========================================================================}

{$C FIXED PRELOAD PERMANENT}

(*{$R-} {$D-} {$S-}*)

unit Hw_16;

interface

uses Wintypes,Winprocs,Classes;

type

  TIRQ      = 0..15;


  TVicHw16 = class(TComponent)

  private

    fOldVector      : Pointer;
    fTerminated     : Boolean;
    fIsIRQSet       : Boolean;
    fMasked         : Boolean;
    fOnHwInterrupt  : TNotifyEvent;
    fIRQ            : TIRQ;
    fHandled        : Longint;
    fBaseAddr       : LongInt;
    fMemoryMapped   : Boolean;
    fSelector       : Word;
    fMemoryPointer  : Pointer;
    fMemorySize     : Word;

  public


    constructor Create(owner:TComponent);
    destructor  Destroy; override;

    function    MapPhysMemory(PhAddr:LongInt; Size:Word):Pointer;
    procedure   UnmapPhysMemory;
    procedure   SetIRQ(HW:TComponent);
    procedure   DestroyIRQ;
    procedure   MaskInterrupt;
    procedure   UnmaskInterrupt;
    procedure   SimulateHwInt;
    property    IsIRQSet      : Boolean      read fIsIRQSet;
    property    Masked        : Boolean      read fMasked;
    property    IRQCounter    : Longint      read fHandled;

    published

    property    OnHwInterrupt : TNotifyEvent read fOnHwInterrupt
                                             write fOnHwInterrupt;
    property    IRQNumber     : TIRQ         read fIRQ write fIRQ;

  end;

{ information about LPT}

const BasePorts:array[0..2] of Word = ($379,$279,$3BC);
      MaxPins  = 17;
type  TPinNumber = 1..MaxPins;

const
      PinsPort: array[TPinNumber] of Byte = (2,0,0,0,0,0,0,0,0,1,1,1,1,2,1,2,2);
      Negative: array[TPinNumber] of Boolean =
           (TRUE,                             {  -STROBE, pin 1,     Port+2,}
            FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE, FALSE,{   DATA,    pins 2..9, Port+0}
            FALSE,                            {   ACKWL,  pin 10,    Port+1}
            TRUE,                             {   BUSY,   pin 11,    Port+1}
            FALSE,                            {   PE,     pin 12,    Port+1}
            FALSE,                            {   SLCT,   pin 13,    Port+1}
            TRUE,                             {  -AUTOFD, pin 14,    Port+2}
            FALSE,                            {  -ERROR,  pin 15,    Port+1}
            FALSE,                            {   INIT,   pin 16,    Port+2}
            TRUE);                            {  -SLCTIN  pin 17,    Port+2}

      MaskPins: array[TPinNumber] of Byte =
           ($01,                              {  -STROBE,  pin 1,     Port+2}
            $01,$02,$04,$08,$10,$20,$40,$80,  {   DATA,    pins 2..9, Port+0}
            $40,                              {  -ACKWL,   pin 10,    Port+1}
            $80,                              {   BUSY,    pin 11,    Port+1}
            $20,                              {   PE,      pin 12,    Port+1}
            $10,                              {   SLCT,    pin 13,    Port+1}
            $02,                              {  -AUTOFD,  pin 14,    Port+2}
            $08,                              {  -ERROR,   pin 15,    Port+1}
            $04,                              {   INIT,    pin 16,    Port+2}
            $08);                             {  -SLCTIN,  pin 17,    Port+2}

type

  TLPTNumber = (LPT1,LPT2,LPT3);
  TVicLPT16 = class(TComponent)
   protected
    fLPTNumber   : TLPTNumber;
    fBasePort    : Word;
    fCommandPort : Word;
    fStatusPort  : Word;
    fDataPorts   : array[0..2] of Byte;
    fIsIRQSet    : Boolean;
    procedure  WritePin(nPin : TPinNumber; Value : Boolean);
    function   ReadPin(nPin : TPinNumber) : Boolean;
    procedure  SetLPT(nLPT:TLPTNumber);

    procedure  SetIRQ(St:Boolean);

  public
    constructor Create(owner:TComponent); override;
    procedure _STROBE;
    function   _ACKWL:Boolean;
    function   BUSY:Boolean;
    function   PE:Boolean;
    function   SLCT:Boolean;
    procedure  _AUTOFD(Flag:Boolean);
    function   _ERROR:Boolean;
    procedure  _INIT;
    procedure  _SLCTIN;
    function   PrintChar(ch:Char):Boolean;
    property   Pin[Index:TPinNumber]:Boolean read ReadPin write WritePin;
  published
    property   LPT         : TLPTNumber read fLPTNumber write SetLPT default LPT1;
    property   BasePort    : Word       read fBasePort;
    property   CommandPort : Word       read fCommandPort;
    property   StatusPort  : Word       read fStatusPort;
    property   IRQEnabled  : Boolean    read fIsIRQSet write SetIRQ default false;
  end;


const
IRQs  : array[TIRQ] of Byte=($08,$09,$0A,$0B,$0C,$0D,$0E,$0F,
                             $70,$71,$72,$73,$74,$75,$76,$77);
Masks : array[TIRQ] of Byte=($01,$02,$04,$08,$10,$20,$40,$80,
                             $01,$02,$04,$08,$10,$20,$40,$80);
Conts : array[TIRQ] of Byte=($21,$21,$21,$21,$21,$21,$21,$21,
                             $A1,$A1,$A1,$A1,$A1,$A1,$A1,$A1);
var
LocHW                        : TVicHw16;


procedure Register;

implementation

procedure GetIntVec(IntNo: Byte; var Vector: Pointer); forward;
procedure SetIntVec(IntNo: Byte; Vector: Pointer);     forward;

{$F+}

procedure IRQProc; interrupt;
begin
  with LocHW do
  begin
    if Assigned(fOnHwInterrupt) then OnHwInterrupt(LocHW);
    Inc(fHandled);
    Port[$20]:=$20;
    if fIRQ>7 then Port[$a0]:=$20;
  end;
end;

constructor TVicHw16.Create(owner:TComponent); {==}
begin
  inherited Create(owner);
  fMasked   := TRUE;
  fIsIRQSet := FALSE;
end;

destructor TVicHw16.Destroy;                   {==}
begin
  if IsIRQSet then
  begin
    if not Masked then MaskInterrupt;
    DestroyIRQ;
  end;
  if fMemoryMapped then UnmapPhysMemory;
  inherited Destroy;
end;

function  TVicHw16.MapPhysMemory(PhAddr:LongInt; Size:Word):Pointer;
begin
  Result:=fMemoryPointer;
  if (PhAddr=fBaseAddr) and (Size=fMemorySize) then Exit;
  if fMemoryMapped then UnmapPhysMemory;
  fMemorySize:=Size;
  fBaseAddr:=PhAddr;
  fMemoryMapped:=TRUE;
  fSelector:=AllocSelector(DSeg);
  SetSelectorBase(fSelector,PhAddr);
  SetSelectorLimit(fSelector,Size);
  fMemoryPointer:=Ptr(fSelector,0);
  Result:=fMemoryPointer;
end;

procedure TVicHw16.UnmapPhysMemory;
begin
  if fMemoryMapped then FreeSelector(fSelector);
  fSelector:=0;
  fMemoryMapped:=FALSE;
  fBaseAddr:=0;
  fMemoryPointer:=NIL;
  fMemorySize:=0;
end;

procedure TVicHw16.SetIRQ(HW:TComponent);
begin
  LocHW:=TVicHW16(HW);
  if (not fIsIRQSet) then
  begin
    Port[Conts[fIRQ]]:=Port[Conts[fIRQ]] or Masks[fIRQ];
    asm cli end;
    GetIntVec(IRQs[fIRQ],fOldVector);
    SetIntVec(IRQs[fIRQ],@IRQProc);
    asm sti end;
    fHandled:=0;
    fIsIRQSet:=TRUE;
    fMasked:=TRUE;
  end;
end;


procedure TVicHw16.DestroyIRQ;
begin
  if (not fIsIRQSet) then Exit;
  if not fMasked then MaskInterrupt;
  asm cli end;
  SetIntVec(IRQs[fIRQ],fOldVector);
  asm sti end;
  fIsIRQSet:=FALSE;
end;

procedure TVicHw16.MaskInterrupt;
begin
  if (not fIsIRQSet) or fMasked then Exit;
  Port[Conts[fIRQ]]:=Port[Conts[fIRQ]] or Masks[fIRQ];
  fMasked:=TRUE;
end;

procedure TVicHw16.UnMaskInterrupt;
begin
  if (not fIsIRQSet) or (not fMasked) then Exit;
  Port[Conts[fIRQ]]:=Port[Conts[fIRQ]] and (not Masks[fIRQ]);
  fMasked:=FALSE;
end;

procedure TVicHw16.SimulateHwInt;
begin
  if (not IsIRQSet) then Exit;
  if Assigned(fOnHwInterrupt) then OnHwInterrupt(Self);
  Inc(fHandled);
end;

constructor TVicLPT16.Create(owner:TComponent);
begin
  inherited Create(owner);
  SetLPT(LPT);
end;

procedure TVicLPT16.SetIRQ(St:Boolean);
begin
  fIsIRQSet:=St;
  if St then fDataPorts[2]:=fDataPorts[2] or  $10
        else fDataPorts[2]:=fDataPorts[2] and $EF;
  Port[fCommandPort]:=fDataPorts[2];
end;

procedure  TVicLPT16.SetLPT(nLPT:TLPTNumber);
type TLPTArray = array[LPT1..LPT3] of Word;
var  PLPTArray :^TLPTArray;
begin
  PLPTArray:=Ptr($40,8);
  if nLPT in [LPT1,LPT2,LPT3] then fBasePort:=PLPTArray^[nLPT]
                              else fBasePort:=PLPTArray^[LPT1];
  fLPTNumber:=nLPT;
  fStatusPort:=fBasePort+1;
  fCommandPort:=fBasePort+2;
  fDataPorts[0]:=0;  fDataPorts[1]:=0;  fDataPorts[2]:=0;
end;

procedure  TVicLPT16.WritePin(nPin : TPinNumber; Value : Boolean);
var data : Byte;
    ofs  : Byte;
begin
  if (nPin>MaxPins) or (nPin=0) then Exit;
  ofs:=PinsPort[nPin];
  data:=fDataPorts[ofs];
  if Byte(Value)<>Byte(Negative[nPin]) then data:=data or MaskPins[nPin]
                                       else data:=data and (not MaskPins[nPin]);
  Port[fBasePort+ofs]:=data;
  fDataPorts[ofs]:=data;
end;

function   TVicLPT16.ReadPin(nPin : TPinNumber) : Boolean;
var data: Byte;
    ofs : Byte;
begin
  Result:=FALSE;
  if (nPin>MaxPins) or (nPin=0) then Exit;
  ofs:=PinsPort[nPin];
  data:=Port[fBasePort+ofs];
  Result:=(data and MaskPins[nPin])<>0;
  if Negative[nPin] then Result:=not Result;
end;

procedure  TVicLPT16._STROBE;
begin
  Pin[1]:=FALSE;
  Pin[1]:=TRUE;
end;

function  TVicLPT16._ACKWL:Boolean;
begin
  Result:=not Pin[10];
end;

function  TVicLPT16.BUSY:Boolean;
begin
  Result:=Pin[11];
end;

function  TVicLPT16.PE:Boolean;
begin
  Result:=Pin[12];
end;

function  TVicLPT16.SLCT:Boolean;
begin
  Result:=not Pin[13];
end;

procedure  TVicLPT16._AUTOFD(Flag:Boolean);
begin
  Pin[14]:=not Flag;
end;

function  TVicLPT16._ERROR:Boolean;
begin
  Result:=not Pin[15];
end;

procedure  TVicLPT16._INIT;
var i : Word;
begin
  Pin[16]:=FALSE;
  for i:=1 to 1000 do;
  Pin[16]:=TRUE;
end;

procedure  TVicLPT16._SLCTIN;
var data : Byte;
begin
  data:=fDataPorts[2] and $F7;
  fDataPorts[2]:=data;
  Port[fBasePort+2]:=data;
end;

function TVicLPT16.PrintChar(ch:Char):Boolean;
var  hh    : array[0..1] of char;
begin
  Result:=FALSE;
  hh[0]:=ch; hh[1]:=#0;
  AnsiToOEM(hh,hh);  ch:=hh[0];
  Port[fBasePort]:=Byte(ch);
  if (not _ERROR) and (not BUSY) then
  begin
    _STROBE;
    Result:=TRUE;
  end;  
 end;

procedure Register;
begin
  RegisterComponents('Drivers',[TVicHW16,TVicLPT16]);
end;


procedure GetIntVec(IntNo: Byte; var Vector: Pointer); assembler;
asm
  MOV	AL,IntNo
  MOV	AH,35H
  INT	21H
  MOV	AX,ES
  LES	DI,Vector
  CLD
  XCHG	AX,BX
  STOSW
  XCHG	AX,BX
  STOSW
end;

procedure SetIntVec(IntNo: Byte; Vector: Pointer); assembler;
asm
  PUSH	DS
  LDS	DX,Vector
  MOV	AL,IntNo
  MOV	AH,25H
  INT	21H
  POP	DS
end;



end.
